#!/bin/bash

mv data/attributes.1.csv data/attributes.csv
mv data/flower_locations.1.csv data/flower_locations.csv
mv data/flowers.1.csv data/flowers.csv
mv data/flowers.garden.1.csv data/flowers.garden.csv
mv data/flowers.pipe.1.csv data/flowers.pipe.csv
mv data/flowers.web.1.csv data/flowers.web.csv
mv data/images.1.csv data/images.csv
mv data/notes.1.csv data/notes.csv
mv data/urls.1.csv data/urls.csv
