#!/bin/bash

test.csv.pl -c 3 -i data/attribute_types.csv

test.csv.pl -c 3 -i data/attributes.csv

test.csv.pl -c 2 -i data/constants.csv

test.csv.pl -c 4 -i data/flower_locations.csv

test.csv.pl -c 6 -i data/flowers.csv

test.csv.pl -c 4 -i data/gardens.csv

test.csv.pl -c 3 -i data/images.csv

test.csv.pl -c 2 -i data/notes.csv

test.csv.pl -c 4 -i data/object_locations.csv

test.csv.pl -c 3 -i data/objects.csv

test.csv.pl -c 3 -i data/properties.csv

test.csv.pl -c 2 -i data/urls.csv
